/**
 * Tocea Softwares.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 23 mai 2012
 * @copyright Copyright (C) 2010-2012 Tocea
 */

package com.tocea.easycoverage.framework;



import java.util.List;

import junit.framework.TestCase;
import junit.framework.TestResult;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.annotations.SkipAutomaticTests;
import com.tocea.easycoverage.framework.api.IClassTest;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes the test instance.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 23 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class SuiteClassTest extends TestCase
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(SuiteClassTest.class);
    private final List<IClassTest>        testBehaviours;
    private final Class<?>                clazz;
    
    
    
    /**
     * Builds a suite class test.
     * 
     * @param _clazz
     * @param _testBehaviours
     */
    @SkipAutomaticTests
    public SuiteClassTest(final Class<?> _clazz, final List<IClassTest> _testBehaviours) {
    
    
        super(_clazz.getCanonicalName());
        this.clazz = _clazz;
        this.testBehaviours = _testBehaviours;
        
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see junit.framework.Test#countTestCases()
     */
    @Override
    public int countTestCases() {
    
    
        return this.testBehaviours.size();
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see junit.framework.Test#run(junit.framework.TestResult)
     */
    @Override
    public void run(final TestResult _result) {
    
    
        _result.startTest(this);
        
        try {
            this.testingClass();
            
            
        } catch (final Throwable e) {
            LOGGER.error("Error testing the class  : {}", this.clazz);
            e.printStackTrace();
            _result.addError(this, e);
        } finally {
            _result.endTest(this);
        }
        
        
    }
    
    
    /**
     * This method performs the test of a class
     */
    public void testingClass() throws WeakImplementationException {
    
    
        final SkipAutomaticTests annotation = this.clazz.getAnnotation(SkipAutomaticTests.class);
        if (annotation != null) {
            LOGGER.debug("\t[SKIP] Test of class {}", this.clazz.getCanonicalName()); //$NON-NLS-1$
            
            return;
        }
        LOGGER.debug("\t[TEST] {}", this.clazz); //$NON-NLS-1$
        for (final IClassTest testBehaviour : this.testBehaviours) {
            LOGGER.debug("\t[TEST] >" + testBehaviour); //$NON-NLS-1$
            testBehaviour.test(this.clazz);
        }
    }
    
    
    @Override
    public String toString() {
    
    
        return "Test of " + this.clazz.getCanonicalName();
    }
}
