/**
 * Tocea Softwares.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 23 mai 2012
 * @copyright Copyright (C) 2010-2012 Tocea
 */

package com.tocea.easycoverage.framework;



import java.util.List;

import junit.framework.TestCase;
import junit.framework.TestResult;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.annotations.SkipAutomaticTests;
import com.tocea.easycoverage.framework.api.IObjectTest;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes the test suite for an object instance.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 23 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class SuiteInstanceTest extends TestCase
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(SuiteInstanceTest.class);
    
    private final List<IObjectTest>       testBehaviours;
    
    private final Object                  object;
    
    
    
    /**
     * Builds a test
     * 
     * @param _testBehaviours
     *            the automatic tests provided
     * @param _object
     *            the object to test
     */
    @SkipAutomaticTests
    public SuiteInstanceTest(final Object _object, final List<IObjectTest> _testBehaviours) {
    
    
        super("Instance of " + _object.getClass().getCanonicalName()); //$NON-NLS-1$
        this.testBehaviours = _testBehaviours;
        this.object = _object;
        
        
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see junit.framework.Test#countTestCases()
     */
    @Override
    public int countTestCases() {
    
    
        return this.testBehaviours.size();
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see junit.framework.Test#run(junit.framework.TestResult)
     */
    @Override
    public void run(final TestResult _result) {
    
    
        _result.startTest(this);
        
        try {
            this.testingInstanceObject();
        } catch (final Throwable e) {
            LOGGER.error("Error testing the object of instance  : {}", this.object.getClass().getCanonicalName());
            e.printStackTrace();
            _result.addError(this, e);
        } finally {
            _result.endTest(this);
        }
    }
    
    
    /**
     * This method performs the test of an instance.
     */
    public void testingInstanceObject() throws WeakImplementationException {
    
    
        final SkipAutomaticTests annotation = this.object.getClass().getAnnotation(SkipAutomaticTests.class);
        if (annotation != null) {
            LOGGER.debug("\t[SKIP] instance of {}", this.object.getClass().getCanonicalName()); //$NON-NLS-1$
            
            return;
        }
        
        LOGGER.debug("\t[TEST] Object '{}' of {}", this.object, this.object.getClass()); //$NON-NLS-1$
        for (final IObjectTest testBehaviour : this.testBehaviours) {
            LOGGER.debug("\t[TEST] >" + testBehaviour); //$NON-NLS-1$
            testBehaviour.test(this.object);
            
        }
        
    }
    
    
    @Override
    public String toString() {
    
    
        return "Test of the instance of " + this.object.getClass().getCanonicalName(); //$NON-NLS-1$
    }
}
