/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework;



import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.api.IAutomaticTest;
import com.tocea.easycoverage.framework.unit.ArrayInvokationTest;
import com.tocea.easycoverage.framework.unit.BijectiveCompareToTest;
import com.tocea.easycoverage.framework.unit.BijectiveEqualsTest;
import com.tocea.easycoverage.framework.unit.GetterSetterInvokationTest;
import com.tocea.easycoverage.framework.unit.MissingToStringTest;
import com.tocea.easycoverage.framework.unit.NPECloneTest;
import com.tocea.easycoverage.framework.unit.NPECompareToTest;
import com.tocea.easycoverage.framework.unit.NPEConstructorTest;
import com.tocea.easycoverage.framework.unit.NPEEqualsTest;
import com.tocea.easycoverage.framework.unit.NPEMethodTest;
import com.tocea.easycoverage.framework.unit.NPEToStringTest;
import com.tocea.easycoverage.framework.unit.NullValueEqualsTest;
import com.tocea.easycoverage.framework.unit.SimpleInstantiationTest;
import com.tocea.easycoverage.framework.unit.SimpleInvokationTest;
import com.tocea.easycoverage.framework.unit.ToStringShouldNeverReturnsNullTest;
import com.tocea.easycoverage.framework.unit.VariableArgumentInvocationTest;



/**
 * This type describes the automatic test
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 12 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */
public final class TestAutomatic
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER          = LoggerFactory.getLogger(TestAutomatic.class);
    
    
    @SuppressWarnings("rawtypes")
    private static final Class[]          AVAILABLE_TESTS = {
            ArrayInvokationTest.class,
            BijectiveCompareToTest.class,
            BijectiveEqualsTest.class,
            GetterSetterInvokationTest.class,
            MissingToStringTest.class,
            NPECloneTest.class,
            NPECompareToTest.class,
            NPEConstructorTest.class,
            NPEEqualsTest.class,
            NPEMethodTest.class,
            NPEToStringTest.class,
            NullValueEqualsTest.class,
            SimpleInstantiationTest.class,
            SimpleInvokationTest.class,
            ToStringShouldNeverReturnsNullTest.class,
            VariableArgumentInvocationTest.class,        };
    
    
    
    /**
     * This method returns the available plugins.
     * 
     * @return the available plugins.
     */
    @SuppressWarnings("unchecked")
    public static Class<IAutomaticTest>[] getAvailablePlugins() {
    
    
        return TestAutomatic.AVAILABLE_TESTS;
    }
    
    
    /**
     * This method returns a new class scanner.
     * 
     * @return a new class scanner.
     */
    public static ClassScanner newClassScanner() {
    
    
        return new ClassScanner();
        
    }
    
    
    /**
     * This method performs the creation of a test harness.
     * 
     * @return the new test harness
     */
    @SuppressWarnings("unchecked")
    public static TestHarness newTestHarness() {
    
    
        return newTestHarness(AVAILABLE_TESTS);
    }
    
    
    /**
     * This method performs the creation of a test harness.
     * 
     * @param _plugins
     *            the automatic tests to launch.
     * @return the new test harness
     */
    public static TestHarness newTestHarness(final Class<? extends IAutomaticTest>... _plugins) {
    
    
        final TestHarness testHarness = new TestHarness();
        for (final Class<? extends IAutomaticTest> automaticTest : _plugins) {
            if (automaticTest == null) {
                continue; // Skip null classes
            }
            try {
                testHarness.addBehaviour(automaticTest.newInstance());
            } catch (final InstantiationException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final IllegalAccessException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final SecurityException e) {
                LOGGER.error(e.getMessage(), e);
                
            }
            
        }
        return testHarness;
    }
    
    
    private TestAutomatic() {
    
    
        super();
        
    }
    
    
    @Override
    public String toString() {
    
    
        return "Component to perform automatic tests";
    }
}
