/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.Method;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.api.IObjectTest;
import com.tocea.easycoverage.framework.utils.Utils;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes a test of the NPE on compareto method.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class BijectiveCompareToTest implements IObjectTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(BijectiveCompareToTest.class);
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedObject
     *            the tested class.
     */
    public BijectiveCompareToTest() {
    
    
        super();
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.api.IObjectTest#test(java.lang.Object)
     */
    public void test(final Object testedObject) throws WeakImplementationException {
    
    
        LOGGER.debug("Processing the method clone to win against NPE faults."); //$NON-NLS-1$
        if (Utils.shouldSkip(testedObject.getClass())) {
            return;
        }
        
        if (testedObject instanceof Comparable) {
            
            try {
                final Method method = testedObject.getClass().getMethod("compareTo", Object.class); //$NON-NLS-1$
                final Object newInstance = method.getParameterTypes()[0].newInstance();
                method.setAccessible(true);
                final int comp1 = (Integer) method.invoke(testedObject, newInstance);
                final int comp2 = (Integer) method.invoke(newInstance, testedObject);
                if (comp1 != -comp2) {
                    
                    throw new WeakImplementationException(
                            "The method compareTo should implements the predicate as compareTo(x,y) == -compareTo(x,y).", this.getClass() //$NON-NLS-1$
                                    .getCanonicalName());
                }
            } catch (final Throwable e) {
                // /
                
            }
        }
    }
    
}
