/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.api.IClassTest;
import com.tocea.easycoverage.framework.utils.Utils;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class MissingToStringTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(MissingToStringTest.class);
    
    
    
    /**
     * Builds a simple invocation test .
     */
    public MissingToStringTest() {
    
    
        super();
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.api.IClassTest#test()
     */
    public void test(final Class<?> _testedClass) throws WeakImplementationException {
    
    
        LOGGER.debug("Processing the method toString() to win against NPE faults."); //$NON-NLS-1$
        if (Utils.shouldSkip(_testedClass.getClass())) {
            return;
        }
        
        if (_testedClass.isInterface() || _testedClass.isEnum() || _testedClass.isAnnotation() || _testedClass.isAnonymousClass()) {
            return;
        }
        boolean hasInstance = false;
        boolean hasStatic = false;
        for (final Method meth : _testedClass.getMethods()) {
            if (!Modifier.isStatic(meth.getModifiers())) {
                hasInstance = true;
                break;
            } else {
                hasStatic = true;
            }
        }
        if (!hasInstance && hasStatic) {
            return;
        }
        try {
            final Method method = _testedClass.getMethod("toString"); //$NON-NLS-1$
            if (method.getDeclaringClass().equals(Object.class)) {
                throw new WeakImplementationException("Missing toString method", MissingToStringTest.class.getCanonicalName()); //$NON-NLS-1$
            }
        } catch (final SecurityException e) {
            //
            
        } catch (final NoSuchMethodException e) {
            //
            
        }
    }
}
