/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.Constructor;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.api.IClassTest;
import com.tocea.easycoverage.framework.utils.ConstructorImplementationException;
import com.tocea.easycoverage.framework.utils.Utils;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class SimpleInstantiationTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(SimpleInstantiationTest.class);
    private int                           excludeMethodsModifiers;
    
    
    
    /**
     * Builds a simple invocation test .
     */
    public SimpleInstantiationTest() {
    
    
        super();
    }
    
    
    /**
     * Returns the excludeMethodsModifiers field.
     * 
     * @return the excludeMethodsModifiers
     */
    public int getExcludeMethodsModifiers() {
    
    
        return this.excludeMethodsModifiers;
    }
    
    
    /**
     * Sets the excludeMethodsModifiers to a new value.
     * 
     * @param excludeMethodsModifiers
     *            the excludeMethodsModifiers to set
     */
    public void setExcludeMethodsModifiers(final int excludeMethodsModifiers) {
    
    
        this.excludeMethodsModifiers = excludeMethodsModifiers;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.api.IObjectTest#test(java.lang.Object)
     */
    public void test(final Class<?> testedClass) throws WeakImplementationException {
    
    
        if (Utils.shouldSkip(testedClass)) {
            return;
        }
        
        
        LOGGER.debug("Processing instantiation of the class {} if any constructors available"); //$NON-NLS-1$
        for (final Constructor<?> constructor : Utils.getAccessibleConstructors(testedClass, this.getExcludeMethodsModifiers())) {
            if (Utils.shouldSkip(constructor) || constructor.getParameterTypes().length > 0) {
                continue;
            }
            
            LOGGER.debug("Testing the constructor {}", constructor); //$NON-NLS-1$
            constructor.setAccessible(true);
            try {
                constructor.newInstance();
            } catch (final Throwable e) {
                throw new ConstructorImplementationException(e, constructor, this.getClass().getCanonicalName());
            }
            constructor.setAccessible(false);
        }
        
        
    }
    
}
