/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.api.IObjectTest;
import com.tocea.easycoverage.framework.utils.Utils;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class ToStringShouldNeverReturnsNullTest implements IObjectTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(ToStringShouldNeverReturnsNullTest.class);
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedObject
     *            the tested class.
     */
    public ToStringShouldNeverReturnsNullTest() {
    
    
        super();
        
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.api.IObjectTest#test(java.lang.Object)
     */
    public void test(final Object testedObject) throws WeakImplementationException {
    
    
        LOGGER.debug("Processing the method toString() to win against NPE faults."); //$NON-NLS-1$
        if (Utils.shouldSkip(testedObject.getClass())) {
            return;
        }
        try {
            if (testedObject.toString() == null) {
                throw new WeakImplementationException("toString() method should never returns null or any exception", this.toString());
            }
        } catch (final Throwable _exception) {
            throw new WeakImplementationException("toString() method should never returns null or any exception", this.toString(),
                    _exception);
        }
        
    }
    
    
    @Override
    public String toString() {
    
    
        return "tostring_must_not_return_null";
    }
}
